PRO fit_rm,lamb2,phi_in,err_in,phi_ini,rm,xi2,flag,democratic=democratic,$
           rmmax=rmmax,rmmin=rmmin,noweight=noweight,warning=warning,$
           no_n_pi=no_n_pi,minf=minf,aips=aips
; lamb2      : Input lambda^2 in in units of m^2 [Number of Frequencies]
; phi_in     : Input vector of phi values        [Number of Frequencies]
; err_in     : Input vector of phi errors        [Number of Frequencies]
; phi_ini    : Resulting initial Phi_0
; rm         : Resulting Rotation Measure in units of rad/m^2
; xi2        : Resultiong xi2
; flag       : flag for success
; democratic : allows to use multiple pixels for solving global n-Pi
; rmmax      : Maximum RM value for n-Pi cone
; rmmin      : Minimum RM value for n-Pi cone
; noweight   : do not use weight fits (not suggested to use !)
; no_n_pi    : do RM-fit without solving global n-Pi (for paches wich
;              imported global n-Pi from sourrounding ...)
; minf       : Sets minimum index for frequency, where global n-Pi is
;              searched (needed for afterburn)
; aips       : calculate rmmin,rmmax from the first two frequencies

   IF NOT(IS_DEF(rmmax)) THEN rmmax=2000.0
   IF NOT(IS_DEF(rmmin)) THEN rmmin=-rmmax
   IF NOT(IS_DEF(warning)) THEN warning=0
   IF NOT(IS_DEF(no_n_pi)) THEN no_n_pi=0
   IF NOT(IS_DEF(minf)) THEN minf=0
   errwght=1.5
   ss=SIZE(phi_in)
   IF ss(0) EQ 1 THEN BEGIN
      nmaps=ss(1)
      npixels=1
   END ELSE BEGIN
      nmaps=ss(1)
      npixels=ss(2)
   END

;   print,"fit_rm: warning=",warning

   y=dblarr(nmaps)
   w=dblarr(nmaps)
   x=dblarr(1,nmaps)
;
   phi_vec=dblarr(nmaps,npixels)
   err_vec=dblarr(nmaps,npixels)
;
   xi2=dblarr(npixels)
   rm=dblarr(npixels)
   phi_ini=dblarr(npixels)
   flag=dblarr(npixels)
   flag(*)=0.
;
   n=intarr(nmaps)
   nmin=intarr(nmaps)
   nmax=intarr(nmaps)

   IF ss(0) EQ 1 THEN BEGIN
      phi_vec(*,0)=phi_in(*)/180.*!Pi
      err_vec(*,0)=err_in(*)/180.*!Pi
   END ELSE BEGIN
      phi_vec(*,*)=phi_in(*,*)/180.*!Pi
      err_vec(*,*)=err_in(*,*)/180.*!Pi
   END

   x(0,*)=lamb2(*)

IF IS_DEF(aips) THEN BEGIN
; find optimal (closest) frequency pair ....
   l2s=SORT(lamb2)
   dmin=1e10
   imin=-1
   FOR i=0,nmaps-2 DO BEGIN
      dtest=lamb2(l2s(i+1))-lamb2(l2s(i))
      IF dtest LT dmin THEN BEGIN
         dmin=dtest
         imin=i
      END
   END
   
   phi0_=phi_vec(l2s(imin),0)
   err0_=err_vec(l2s(imin),0)
   lamb20_=lamb2(l2s(imin))
   phi1_=phi_vec(l2s(imin+1),0)
   err1_=err_vec(l2s(imin+1),0)
   lamb21_=lamb2(l2s(imin+1))

   IF ABS(phi0_-phi1_) GT !Pi/2 THEN phi1_=phi1_+FLOOR((phi0_-phi1_)/!Pi+0.5)*!Pi
   IF lamb20_ GT lamb21_ THEN BEGIN
      swap,lamb20_,lamb21_
      swap,phi0_,phi1_
      swap,err0_,err1_
   END
   rmmax=(phi0_-phi1_+errwght*(err0_+err1_))/(lamb20_-lamb21_)
   rmmin=(phi0_-phi1_-errwght*(err0_+err1_))/(lamb20_-lamb21_)
   IF npixels GT 0 THEN print,'Infered RMmin/Rmmax: ',rmmin,rmmax

END

IF no_n_pi EQ 0 THEN BEGIN

   nmin(0)=0
   nmax(0)=0
   n(0)=0

   ntot=1L
   FOR in=1,nmaps-1 DO BEGIN
      IF in GE minf THEN BEGIN
         IF lamb2(in) GT lamb2(0) THEN BEGIN
            nmax(in)=FLOOR((phi_vec(0,0)+rmmax*(lamb2(in)-lamb2(0))+$
                            errwght*(err_vec(in,0)+err_vec(0,0))-phi_vec(in,0))/!Pi)
            nmin(in)= CEIL((phi_vec(0,0)+rmmin*(lamb2(in)-lamb2(0))-$
                            errwght*(err_vec(in,0)+err_vec(0,0))-phi_vec(in,0))/!Pi)
         END ELSE BEGIN
            nmin(in)= CEIL((phi_vec(0,0)-rmmax*(lamb2(0)-lamb2(in))-$
                            errwght*(err_vec(in,0)+err_vec(0,0))-phi_vec(in,0))/!Pi)
            nmax(in)=FLOOR((phi_vec(0,0)-rmmin*(lamb2(0)-lamb2(in))+$
                            errwght*(err_vec(in,0)+err_vec(0,0))-phi_vec(in,0))/!Pi)
         END
         IF nmin(in) GT nmax(in) THEN BEGIN
            ntmp=nmax(in)
            nmax(in)=nmin(in)
            nmin(in)=ntmp
         END
      END ELSE BEGIN
         nmin(in)=0
         nmax(in)=0
      END

      n(in)=nmin(in)
      ntot=ntot*(nmax(in)-nmin(in)+1)
;      print,nmin(in),nmax(in),ntot
   END

   xi2min=1.0e10
   xi2min2=1.0e10

   rmmerk=dblarr(ntot)
   phi0merk=dblarr(ntot)
   xi2merk=dblarr(ntot)

   janz=1
   IF IS_DEF(democratic) THEN BEGIN
      IF democratic EQ 1 THEN janz_max=100 ELSE janz_max=democratic
      janz=MIN([janz_max,npixels])
      print,'finding global n-Pi over ',janz,' Pixel ...'
   END

   FOR i=0L,ntot-1 DO BEGIN
      Chisqsum=0.
      FOR j=0,janz-1 DO BEGIN
         FOR f=0,nmaps-1 DO BEGIN
            y(f)=phi_vec(f,j)+n(f)*!Pi
         END

         w(*)=err_vec(*,j)
         IF IS_DEF(noweight) THEN $
            yy=REGRESS(x,y,const=const,Chisq=Chisq) $
         ELSE $
            yy=REGRESS(x,y,const=const,Chisq=Chisq,MEASURE_ERRORS=w)

         IF j EQ 0 THEN BEGIN
            rmmerk(i)=yy(0)
            phi0merk(i)=const
         END
         Chisqsum=Chisqsum+Chisq
      END
      xi2merk(i)=Chisqsum/janz

      IF xi2merk(i) LT xi2min THEN BEGIN
         nbest=n
         xi2min=xi2merk(i)
      END

      n(1)=n(1)+1
      FOR f=1,nmaps-2 DO BEGIN
         IF n(f) GT nmax(f) THEN BEGIN
            n(f)=nmin(f)
            n(f+1)=n(f+1)+1
         END
      END
   END

   IF ntot GT 1 THEN BEGIN
      iis=SORT(xi2merk)
      IF (xi2merk(iis(1))/2. LT xi2merk(iis(0))) AND (npixels GT 1) THEN BEGIN
         print,'WARNING: two similar best fits  !!!'
         print,'         RM1 =',rmmerk(iis(0)),'  Xi2',xi2merk(iis(0))
         print,'         RM2 =',rmmerk(iis(1)),'  Xi2=',xi2merk(iis(1))
      END
      flag(*)=xi2merk(iis(0))/xi2merk(iis(1))
   END ELSE BEGIN
      flag(*)=0.0
   END
   start_pix=0L
END ELSE BEGIN
      start_pix=0L
      n(*)=0
      nbest=n
END


   IF npixels GT start_pix THEN BEGIN
      FOR j=start_pix,npixels-1L DO BEGIN
         FOR f=0,nmaps-1 DO BEGIN
            y(f)=phi_vec(f,j)+nbest(f)*!Pi
            phi_in(f,j)=phi_vec(f,j)*180/!Pi+nbest(f)*180
         END

         w(*)=err_vec(*,j)
         IF IS_DEF(noweight) THEN $
            yy=REGRESS(x,y,const=const,Chisq=Chisq) $
         ELSE $
            yy=REGRESS(x,y,const=const,Chisq=Chisq,MEASURE_ERRORS=w)

         rm(j)=yy(0)
         phi_ini(j)=const*180/!Pi
         xi2(j)=Chisq

         IF abs(rm(0)) GT rmmax THEN BEGIN
            IF warning GE 1 THEN BEGIN
               print,'WARNING: RM bigger than RM_max !!!!'
               print,'         RM1 =',rm(0),'  Xi2=',xi2(0)
            END
;            flag(j)=flag(j)+2.
         END

      END
   END
;   stop
END
