;################################################
; define the location and names of maps available
;################################################
; Input:
; => sourec      : Name of Source
; Output:
; <= found       : flag if source exist
; Optional:
; <= path        : path to data
; <= ix0min       : defines region of interest
; <= ix0max       : defines region of interest
; <= iy0min       : defines region of interest
; <= iy0max       : defines region of interest
; <= subset       : allows to define different flavours of one dataset

; basically you have to give the filemanes holding for the polarisation angles
; and the errors in the polarisation angles.
;
; For simplicity you can also give a common path to your data
;
; you can copy the IF block several times to host different data sets.
;
; Beside that corresponding files should eb given in the same order,
; there is no need to have a certain order. Note that in the case you
; use the /apis option, it usese the first two given frequencies to
; solve the n-pi, so in that case the order is important.
;
; There is also no principial limit in the number of frequencies once
; can use. The number of frequencies is infered from the number of
; entries in the phi_pol variable.
;
; Please go carefully through the following lines and make the changes
; necesarry for you.

pro get_dataset,source,found,frq,phi_pol,phi_pol_err,group,$
                ix0min=ix0min,ix0max=ix0max,iy0min=iy0min,iy0max=iy0max,path=path,$
                phi_err_tre=phi_err_tre,frq_need=frq_need,subset=subset,err_mul=err_mul
   found=0
   path='./'

; eventuell rausnehmen
;@mydataset

; Uncomment the path statement and give your path if you want.
; has to end with an '/' !
;   path='/data/what_ever_you_want/'

; Replace MYSOURCE by a name of your coice. This string has later to be
; given when calling pacerman
   IF source EQ 'MYSOURCE' THEN BEGIN
; Maybe you want to organize your data in subdirs ? Then uncomment and fill.
      path=path+'MYDATASUBDIR/'

; Give the frequencies of your observations. If you don't give it, it
; will be read from the CRVAL3 header keyword in the fits files
      frq=        [4535.0,$                              ; Frequencys in MHz
                   4885.0,$
                   8085.0,$
                   8465.0]

; Give the files containing the polarisation angle
      phi_pol=    ["_6CMIF2.POLA.FITS",$             ; polarisation angle
                   "_6CMIF1.POLA.FITS",$
                   "_3CMIF1.POLA.FITS",$
                   "_3CMIF2.POLA.FITS"]

; Give the files containing the error of polarisation angle
      phi_pol_err=["_6CMIF2.POLAE.FITS",$         ; error of polarisation angle
                   "_6CMIF1.POLAE.FITS",$
                   "_3CMIF1.POLAE.FITS",$
                   "_3CMIF2.POLAE.FITS"]

; optional definitions

; To speed up, you can restrict the used aerea within the map (if the
; source is smaller than your map)
;      ix0min=400
;      ix0max=800
;      iy0min=200
;      iy0max=600

; One can group frequencies by giving the same value to different frequencies 
; in the 'group' array. The needed number of frequencies then is interpreted 
; as having at least num_frq frequencies (parameter to pacerman) out of 
; DIFFERENT groups. 
;      group=[1,1,2,2]

; One can force a frequency to be a part of the fit by setting '1' in the 
; frq_need array. 
;      frq_need=[0,0,0,0]

      found=1
   END

   IF NOT(IS_DEF(group)) THEN BEGIN
      ss=SIZE(phi_pol)
      group=INDGEN(ss(1))+1
   END

; This tryes to infere the frequencies, if not given above
   IF ((found EQ 1) AND (NOT(IS_DEF(frq)))) THEN BEGIN
      ss=SIZE(phi_pol)
      frq=dblarr(ss(1))
      FOR i=0,ss(1)-1 DO BEGIN
         myhead=headfits(path+phi_pol(i))
         myfrq=SXPAR(myhead,'CRVAL3',dummy)
         frq(i)=myfrq/1.0e9
         print,'Map ',i,': frq=',frq(i),' GHz'
      END
   END
end



