;################################################
; Load data and constructing a proper mask
;################################################
; Input:
; => phi_pol      : array for the filenames
; => phi_pol_err  : array for the filenames
; Output:
; <= map_stack    : array for maps containing the angles
; <= err_stack    : array for errors of the angles
; <= mask_stack   : array for mask
; <= mask         : array for final mask
; Optional:
; => path         : path to data
; => ix0min       : defines region of interest
; => ix0max       : defines region of interest
; => iy0min       : defines region of interest
; => iy0max       : defines region of interest
; => nmin_frq     : min numbers of frequencies required
; => show         : flag to display progress

pro make_mask,frq,map_stack,err_stack,mask_stack,mask,$
              phi_pol_file=phi_pol_file,phi_pol_err=phi_pol_err,i_pol=i_pol,$
              path=path,show=show,$
              ix0min=ix0min,ix0max=ix0max,iy0min=iy0min,iy0max=iy0max,nmin_frq=nmin_frq,$
              phi_err_tre=phi_err_tre,header=header,err_mul=err_mul

   IF NOT(IS_DEF(path)) THEN path='../'

   ss=SIZE(phi_pol_file)
   nfr=ss(1)

   IF NOT(IS_DEF(err_mul)) THEN BEGIN
      err_mul=FLTARR(nfr)
      err_mul(*)=1.0
   END

   IF NOT(IS_DEF(nmin_frq)) THEN nmin_frq=nfr

   tmp=readfits(path+phi_pol_file(0),header)
   ssm=size(tmp)
   map_stack=fltarr(ss(1),ssm(1),ssm(2))
   err_stack=fltarr(ss(1),ssm(1),ssm(2))
   mask_stack=fltarr(ss(1),ssm(1),ssm(2))
   mask_tmp=fltarr(ssm(1),ssm(2))

   IF IS_DEF(mask_file) THEN BEGIN
      print,'Reading Mask File ...'
      mask=readfits(path+mask_file)
      IF NOT(IS_DEF(mask_tresh)) THEN mask_tresh=1.0
      jj=WHERE(mask LT mask_tresh)
      mask(jj)=0.0
      jj=WHERE(mask GT 0.0)
      mask(jj)=1.0
   END ELSE BEGIN
      print,'Constructing Mask ...',phi_err_tre
      mask=fltarr(ssm(1),ssm(2))
      mask(*,*)=0.0
   END

   noisebmap=fltarr(ssm(1),ssm(2))
   FOR i=0,nfr-1 DO BEGIN
      print,'Reading files for ',frq(i),' MHz'
      IF i EQ 0 THEN tmp1=tmp ELSE tmp1=readfits(path+phi_pol_file(i))
      map_stack(i,*,*)=tmp1(*,*)
      tmp2=readfits(path+phi_pol_err(i))
      err_stack(i,*,*)=tmp2(*,*)*err_mul(i)

      print,'Constructing Mask from Polarisation Angles ...'
      mask_tmp(*,*)=0.0
      jj=WHERE((tmp2 LT phi_err_tre(i)) AND (tmp2 GE 0.) AND (ABS(tmp1) GT -1))
      print,min(tmp1(jj)),max(tmp1(jj)),min(tmp2(jj)),max(tmp2(jj))
      mask_tmp(jj)=1.0
      mask_stack(i,*,*)=mask_tmp(*,*)
      ssj=SIZE(jj)
      IF ssj(0) GT 0 THEN BEGIN
         print,'New frequency has masked to ',ssj(1),' pixel'
;         mask(*,*)=0.0
         mask(jj)=mask(jj)+1.0
      END
;      IF IS_DEF(show) THEN mytv,mask
   END

 ;  IF NOT(IS_DEF(ix0min)) THEN BEGIN
 ;     mytv,mask
 ;     print,'Please mark region of interest ...'
 ;     point,x0=x00,y0=y00
 ;     point,x0=x01,y0=y01
 ;     ix0min=MIN([x00,x01])
 ;     ix0max=MAX([x00,x01])
 ;     iy0min=MIN([y00,y01])
 ;     iy0max=MAX([y00,y01])
 ;  END

  IF IS_DEF(ix0min) THEN mask(0:ix0min,*)=0.0
   IF IS_DEF(ix0max) THEN mask(ix0max:ssm(1)-1,*)=0.0
   IF IS_DEF(iy0min) THEN mask(*,0:iy0min)=0.0
   IF IS_DEF(iy0max) THEN mask(*,iy0max:ssm(2)-1)=0.0

   jj=WHERE(mask GE nmin_frq)
;   mask(*,*)=0.0
;   mask(jj)=1.0
   IF IS_DEF(show) THEN mytv,mask
;stop
end
